# Time-stamp: <2008-07-10 14:48:14 Tao Liu>

"""Module Description

Copyright (c) 2008 Tao Liu <taoliu@jimmy.harvard.edu>

This code is free software; you can redistribute it and/or modify it
under the terms of the BSD License (see the file COPYING included with
the distribution).

@status:  experimental
@version: $Revision$
@author:  Tao Liu
@contact: taoliu@jimmy.harvard.edu
"""

# ------------------------------------
# python modules
# ------------------------------------
from math import exp
# ------------------------------------
# constants
# ------------------------------------
LSTEP = 200
EXPTHRES = exp(LSTEP)
EXPSTEP  = exp(-LSTEP)
# ------------------------------------
# Misc functions
# ------------------------------------

import math

def normal_01_cdf ( x ):
    """NORMAL_01_CDF evaluates the Normal 01 CDF.
    """
    a1 = 0.398942280444
    a2 = 0.399903438504
    a3 = 5.75885480458
    a4 = 29.8213557808
    a5 = 2.62433121679
    a6 = 48.6959930692
    a7 = 5.92885724438
    b0 = 0.398942280385
    b1 = 3.8052E-08
    b2 = 1.00000615302
    b3 = 3.98064794E-04
    b4 = 1.98615381364
    b5 = 0.151679116635
    b6 = 5.29330324926
    b7 = 4.8385912808
    b8 = 15.1508972451
    b9 = 0.742380924027
    b10 = 30.789933034
    b11 = 3.99019417011

    if abs ( x ) <= 1.28:
        y = 0.5 * x * x
        q = 0.5 - abs ( x ) * ( a1 - a2 * y / ( y + a3 - a4 / ( y + a5 + a6 / ( y + a7 ) ) ) )
    elif abs ( x ) <= 12.7:
        y = 0.5 * x * x

        q = exp ( - y ) * b0 / ( abs ( x ) - b1 \
                                + b2 / ( abs ( x ) + b3 \
                                + b4 / ( abs ( x ) - b5 \
                                + b6 / ( abs ( x ) + b7 \
                                - b8 / ( abs ( x ) + b9 \
                                + b10 / ( abs ( x ) + b11 ) ) ) ) ) )
    else :
        q = 0.0
    #
    #  Take account of negative X.
    #
    if x < 0.0:
        cdf = q
    else:
        cdf = 1.0 - q

    return cdf

def normal_cdf_inv(p, mu = None, sigma2 = None, lower=True):

    upper = not lower
    if p < 0 or p > 1:
        raise Exception("Illegal argument %f for qnorm(p)." % p)
    
    split = 0.42
    a0 = 2.50662823884
    a1 = -18.61500062529
    a2 = 41.39119773534
    a3 = -25.44106049637
    b1 = -8.47351093090
    b2 = 23.08336743743
    b3 = -21.06224101826
    b4 = 3.13082909833
    c0 = -2.78718931138
    c1 = -2.29796479134
    c2 = 4.85014127135
    c3 = 2.32121276858
    d1 = 3.54388924762
    d2 = 1.63706781897
    q = p - 0.5
    
    r = 0.0
    ppnd = 0.0
    
    if abs(q) <= split:
        r = q * q
        ppnd = q * (((a3 * r + a2) * r + a1) * r + a0) / ((((b4 * r + b3) * r + b2) * r + b1) * r + 1)
    else:
        r = p
        if q > 0:
            r = 1 - p
        
        if r > 0:
            r = math.sqrt(- math.log(r))
            ppnd = (((c3 * r + c2) * r + c1) * r + c0) / ((d2 * r + d1) * r + 1)
            
            if q < 0:
                ppnd = - ppnd
        else:
            ppnd = 0
            
    if upper:
        ppnd = - ppnd
    
    if mu != None and sigma2 != None:
        return ppnd * math.sqrt(sigma2) + mu
    else:
        return ppnd

def normal_cdf (z, mu = 0.0, sigma2 = 1.0, lower=True):
    
    upper = not lower

    z = (z - mu) / math.sqrt(sigma2)
    
    ltone = 7.0
    utzero = 18.66
    con = 1.28
    a1 = 0.398942280444
    a2 = 0.399903438504
    a3 = 5.75885480458
    a4 = 29.8213557808
    a5 = 2.62433121679
    a6 = 48.6959930692
    a7 = 5.92885724438
    b1 = 0.398942280385
    b2 = 3.8052e-8
    b3 = 1.00000615302
    b4 = 3.98064794e-4
    b5 = 1.986153813664
    b6 = 0.151679116635
    b7 = 5.29330324926
    b8 = 4.8385912808
    b9 = 15.1508972451
    b10 = 0.742380924027
    b11 = 30.789933034
    b12 = 3.99019417011

    y = 0.0
    alnorm = 0.0
    
    if z < 0:
        upper = not upper
        z = - z
    
    if z <= ltone or upper and z <= utzero:
        y = 0.5 * z * z
        if z > con:
            alnorm = b1 * math.exp(- y) / (z - b2 + b3 / (z + b4 + b5 / (z - b6 + b7 / (z + b8 - b9 / (z + b10 + b11 / (z + b12))))))
        else:
            alnorm = 0.5 - z * (a1 - a2 * y / (y + a3 - a4 / (y + a5 + a6 / (y + a7))))
    else:
        alnorm = 0.0
    if not upper:
        alnorm = 1.0 - alnorm
    return alnorm

def poisson_cdf (n, lam,lower=True):
    """Poisson CDF evaluater.

    This is a more stable CDF function. It can tolerate large lambda
    value. While the lambda is larger than 700, the function will be a
    little slower.

    Parameters:
    n     : your observation
    lam   : lambda of poisson distribution
    lower : if lower is False, calculate the upper tail CDF
    """
    k = int(n)
    if lam <= 0.0:
        raise Exception("Lambda must > 0")

    if lower:
        if lam > 700:
            return __poisson_cdf_large_lambda (k, lam)
        else:
            return __poisson_cdf(k,lam)
    else:
        if lam > 700:
            return __poisson_cdf_Q_large_lambda (k, lam)
        else:
            return __poisson_cdf_Q(k,lam)

def __poisson_cdf (k,a):
    """Poisson CDF For small lambda. If a > 745, this will return
    incorrect result.

    """
    if k < 0:
        return 0                        # special cases
    next = exp( -a )
    cdf = next
    for i in xrange(1,k+1):
        last = next
        next = last * a / i
        cdf = cdf + next
    if cdf > 1:
        return 1
    else:
        return cdf
    
def __poisson_cdf_large_lambda ( k,a ):
    """Slower poisson cdf for large lambda.
    
    """
    if k < 0:
        return 0                        # special cases
    num_parts = int(a/LSTEP)
    last_part = a % LSTEP
    lastexp = exp(-last_part)
    next = EXPSTEP
    num_parts -= 1
    cdf = next
    for i in xrange(1,k+1):
        last = next
        next = last * a / i
        cdf = cdf + next
        if next > EXPTHRES or cdf > EXPTHRES:
           if num_parts>=1:
               cdf *= EXPSTEP
               next *= EXPSTEP
               num_parts -= 1
           else:
               cdf *= lastexp
               lastexp = 1

    for i in xrange(num_parts):
        cdf *= EXPSTEP
    cdf *= lastexp
    return cdf

def __poisson_cdf_Q (k,a):
    """internal Poisson CDF evaluater for upper tail with small
    lambda.

    """
    if k < 0:
        return 1                        # special cases
    next = exp( -a )

    for i in xrange(1,k+1):
        last = next
        next = last * a / i

    cdf = 0
    i = k+1
    while next >0:
        last = next
        next = last * a / i
        cdf += next
        i+=1
    return cdf

def __poisson_cdf_Q_large_lambda (k,a):
    """Slower internal Poisson CDF evaluater for upper tail with large
    lambda.
    
    """
    if k < 0:
        return 1                        # special cases
    num_parts = int(a/LSTEP)
    last_part = a % LSTEP
    lastexp = exp(-last_part)
    next = EXPSTEP
    num_parts -= 1

    for i in xrange(1,k+1):
        last = next
        next = last * a / i
        if next > EXPTHRES:
           if num_parts>=1:
               next *= EXPSTEP
               num_parts -= 1
           else:
               cdf *= lastexp
               lastexp = 1
    cdf = 0
    i = k+1
    while next >0:
        last = next
        next = last * a / i
        cdf += next
        i+=1
        if next > EXPTHRES or cdf > EXPTHRES:
           if num_parts>=1:
               cdf *= EXPSTEP
               next *= EXPSTEP
               num_parts -= 1
           else:
               cdf *= lastexp
               lastexp = 1

    for i in xrange(num_parts):
        cdf *= EXPSTEP
    cdf *= lastexp
    return cdf

def poisson_cdf_inv ( cdf, lam, maximum=1000):
    """inverse poisson distribution.

    cdf : the CDF
    lam : the lambda of poisson distribution

    note: maxmimum return value is 1000
    and lambda must be smaller than 740.
    """
    assert lam < 740
    if cdf < 0 or cdf > 1:
        raise Exception ("CDF must >= 0 and <= 1")
    elif cdf == 0:
        return 0
    sum2 = 0
    newval = exp( -lam )
    sum2 = newval
#     if cdf <= sum2:
#         return i

    for i in xrange(1,maximum+1):
        sumold = sum2
#         if i == 0:
#             newval = exp( -a )
#             if newval==0:
#                 newval = 4.9406564584124654e-324
#             sum2 = newval
#         else:
        last = newval
        newval = last * lam / i
        sum2 = sum2 + newval
        if sumold <= cdf and cdf <= sum2:
            return i
    
    return maximum

def poisson_cdf_Q_inv ( cdf, lam, maximum=1000):
    """inverse poisson distribution.

    cdf : the CDF
    lam : the lambda of poisson distribution

    note: maxmimum return value is 1000
    and lambda must be smaller than 740.
    """
    assert lam < 740
    if cdf < 0 or cdf > 1:
        raise Exception ("CDF must >= 0 and <= 1")
    elif cdf == 0:
        return 0
    sum2 = 0
    newval = exp( -lam )
    sum2 = newval

    for i in xrange(1,maximum+1):
        sumold = sum2
        last = newval
        newval = last * lam / i
        sum2 = sum2 + newval
        if sumold <= cdf and cdf <= sum2:
            return i
    
    return maximum

def poisson_pdf ( k, a ):
    """Poisson PDF.

    PDF(K,A) is the probability that the number of events observed in
    a unit time period will be K, given the expected number of events
    in a unit time.
    """
    if a <= 0:
        return 0
    return exp(-a) * pow (a, k) / factorial (k)
    

def binomial_coef (n,k):
    """BINOMIAL_COEF computes the Binomial coefficient C(N,K)

    n,k are integers.
    """
    mn = min (k, n-k)
    if mn < 0:
        return 0
    elif mn == 0:
        return 1
    else:
        mx = max(k,n-k)
        cnk = float(mx+1)
        for i in xrange(2,mn+1):
            cnk = cnk * (mx+i) / i
    return cnk

def binomial_cdf (x, a, b):
    """ BINOMIAL_CDF compute the binomial CDF.

    CDF(x)(A,B) is the probability of at most X successes in A trials,
    given that the probability of success on a single trial is B.
    """
    if x < 0:
        return 0
    elif a < x:
        return 1
    elif b == 0:
        return 1
    elif b == 1:
        return 0
    else:
        cdf = 0
        for j in xrange(0,x+1):
            cnk = binomial_coef (a,j)
            pr = cnk*pow(b,j)*pow(1-b,a-j)
            cdf = cdf + pr
        return cdf

def binomial_cdf_inv ( cdf, a, b ):
    """BINOMIAL_CDF_INV inverts the binomial CDF.

    """
    if cdf < 0 or cdf >1:
        raise Exception("CDF must >= 0 or <= 1")
    cdf2 = 0
    a = int(a)
    for x in xrange(0,a+1):
        pdf = binomial_pdf (x,a,b)
        cdf2 = cdf2 + pdf
        if cdf < cdf2:
            return x
    return x

def binomial_pdf ( x, a, b ):
    """binomial PDF.

    
    """
    if a < 1:
        return 0
    elif x < 0 or a < x:
        return 0
    elif b == 0:
        if x == 0:
            return 1
        else:
            return 0
    elif b == 1:
        if x == a:
            return 1
        else:
            return 0
    else:
        cnk = binomial_coef ( a, x )
        return ( cnk * pow(b,x) * pow (1-b,a-x) )



def facotrial (n):
    """N!.
    
    """
    if n < 0:
        return 0
    fact = 1
    for i in xrange(2,n+1):
        fact = fact * i
    return fact


# ------------------------------------
# Main function
# ------------------------------------
def test():
    pass



if __name__ == '__main__':
    test()



#!/usr/bin/python
#Last-modified: 18 Mar 2014 03:54:01 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <yfwang0405@gmail.com>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: yfwang0405@gmail.com

# ------------------------------------
# python modules
# ------------------------------------

# python packages
import os
import re
import sys
import copy
import time
import numpy
import bisect
import random
import tempfile
from itertools import izip
from subprocess import Popen, PIPE
#from scipy.cluster.vq import kmeans2
from scipy.stats import fisher_exact
from sklearn.cluster import KMeans

# custom packages
import wRNA

# ------------------------------------
# constants
# ------------------------------------

debug = False
rctable = {'A':'U','U':'A','C':'G','G':'C'}

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

class Fasta(object):
    '''
    Fasta format
        >YIL140W
        AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU
    '''
    def __init__(self,name,seq):
        ''' Initiation. '''
        self.name = name
        self.seq = seq.upper()
    def __len__(self):
        ''' length of sequence. '''
        return len(self.seq)
    def __str__(self):
        ''' Fasta in string. '''
        return ">{0}\n{1}".format(self.name,self.seq)
    def rc(self):
        '''
        Do reverse complement of self.seq. No returns. 
        '''
        self.seq = ''.join([rctable(c) for c in self.seq])

class PARS(Fasta):
    '''
    PARS format for storing sequencing depth.
    Example file:
        >tE-UUC-I
        UCCGAUAUAGUGUAACGGCUAUCACAUCACGCUUUCACCGUGGAGACCGGGGUUCGACUCCCCGUAUCGGAG
        27023.0;99.0;16.0;2.0;2.0;56.0;25.0;58.0;27.0;28.0;41.0;51.0;138.0;220.0;37.0;50.0;52.0;12.0;8.0;43.0;23.0;47.0;74.0;34.0;19.0;31.0;17.0;27.0;33.0;36.0;13.0;7.0;19.0;371.0;12.0;146.0;45.0;22.0;31.0;22.0;34.0;46.0;34.0;16.0;4.0;1.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0
        3187.0;13.0;27.0;9.0;0.0;18.0;17.0;15.0;13.0;15.0;16.0;13.0;43.0;29.0;17.0;34.0;21.0;5.0;5.0;12.0;5.0;9.0;25.0;7.0;9.0;13.0;7.0;533.0;55.0;22.0;11.0;4.0;5.0;79.0;2.0;26.0;17.0;18.0;19.0;20.0;15.0;49.0;33.0;20.0;5.0;1.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0;0.0
    '''
    def __init__(self, name, seq, loops = [], stems = []):
        ''' Initiation. '''
        super(PARS,self).__init__(name,seq)
        if len(loops) == 0:
            self.loops = numpy.zeros(len(seq),dtype=float)
        else:
            self.loops = numpy.array(loops,dtype=float)
        if len(stems) == 0:
            self.stems = numpy.zeros(len(seq),dtype=float)
        else:
            self.stems = numpy.array(stems,dtype=float)
    def __str__(self):
        ''' string of PARS format. '''
        return ">{0}\n{1}\n{2}\n{3}".format(self.name, self.seq, ';'.join([str(round(i,2)) for i in self.loops]), ";".join([str(round(i,2)) for i in self.stems]))
    def normalize(self, sratio = 1, vratio = 1, ntrim = 5):
        ''' Normalization. '''
        self.loops *= sratio
        self.stems *= vratio
        if ntrim == 0:
            return
        self.loops[0:ntrim] = 0
        self.stems[0:ntrim] = 0
        self.loops[-ntrim:] = 0
        self.stems[-ntrim:] = 0
        return
    def rc(self):
        ''' Reverse complement. '''
        super(PARS,self).rc()
        self.loops = self.loops[::-1]
        self.stems = self.stems[::-1]
        return
    def toFastC(self, sthreshold = 2, vthreshold = -2):
        ''' Calculate constraints from PARS depth. '''
        pars = numpy.log2((self.loops+1)/(self.stems+1))
        constraints = numpy.repeat('.', len(self.seq))
        constraints[numpy.where(pars >  sthreshold)] = 'x'
        constraints[numpy.where(pars <  vthreshold)] = '|'
        return FastC(self.name, self.seq, constraints.tostring())
    def toFastCByFisher(self, Ssum, Vsum, threshold = 0.05):
        ''' Fisher exact test to calculate the pvalue and then calculate fdr. '''
        SVsum = (Ssum+Vsum)/2.
        pvalues = numpy.apply_along_axis(lambda x: fisher.pvalue(x[0],Ssum-x[0],x[1],Vsum-x[1]),1,zip(self.loops,self.stems))

class FastC(Fasta):
    '''
    FastC format for RNA structure prediction.
    Example file:
        >YIL140W
        AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU
        .|xx.|.x..||.|...xx.x.xx...||.x|..||...|..........
    Note:
        '.' means free to fold
        'x' means loops
        '|' means stems

    '''
    def __init__(self, name, seq, constraints=None):
        super(FastC,self).__init__(name,seq)
        self.constraints = constraints
        if constraints is None:
            self.constraints = '.'*len(self.seq)
    def __str__(self):
        ''' FastC string. '''
        return ">{0}\n{1}\n{2}".format(self.name,self.seq,self.constraints)

class FastS(Fasta):
    ''' 
    RNA structure.
    Example: Multiple structures are supported.
        >YIL140W
        AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU
        .......((((((((............)))).))))..............  (-6.0)
        .......(((((((..............))).))))..............  (-4.0)
    '''
    def __init__(self, name, seq, structures, scores=None):
        '''
        Initiation.

        Parameters:
            name: string
                Name of sequence.
            seq: string
                RNA sequences.
            structures: string or list of strings
                RNA structure in Dot-Bracket format.
            scores: None, float or list of float
                A score of each structure. Could be the MFE score, the percentage or any other scores.
        Returns:
            fs: FastS object
                FastS object
        '''
        super(FastS,self).__init__(name,seq)
        if isinstance(structures,basestring):
            self.structures = [structures]
        else:
            self.structures = [st for st in structures]
        if scores is None:
            self.scores = [1./len(self.structures)] * len(self.structures)
        elif isinstance(scores,int) or isinstance(scores,float):
            self.scores = [float(scores)]
        else:
            self.scores = [float(sc) for sc in scores]
    def __str__(self):
        ''' FastS string. '''
        lstr = ">{0}\n{1}".format(self.name,self.seq)
        for st, sc in izip(self.structures, self.scores):
            lstr += "\n{0}\t({1})".format(st, round(sc,3))
        return lstr

class IO(object):
    ''' Process raw data to FastC format. '''
    def readDataToPARS(fafile,S,V,maxlen=300,ntrim=5,threshold = 0.05, prefix=None):
        ''' read fasta, S1 and V1 files and normalize by read number. '''
        # print parameters
        print >>sys.stderr, "Parameters:"
        print >>sys.stderr, "\tFasta file:", fafile
        print >>sys.stderr, "\tS1 file:", S
        print >>sys.stderr, "\tV1 file:", V
        print >>sys.stderr, "\tMaximum sequence length:", maxlen > 0 and maxlen or "no-limit"
        print >>sys.stderr, "\tTrim {0} nt at both ends".format(ntrim)
        print >>sys.stderr, "\tThreshold for constraints:", threshold
        if prefix is None:
            prefix = os.path.splitext(os.path.basename(fafile))[0]
        print >> sys.stderr, "\tOutput file prefix:", prefix
        print >>sys.stderr

        # read S1 file
        print >>sys.stderr, "Reading {0}".format(S)
        starttime=time.clock()
        Ssum = 0
        S1 = {}
        i0 = 0
        with open(S) as fh:
            for s in fh:
                i0 += 1
                if i0 % 100 == 0:
                    print >> sys.stderr, "Prcessed ",i0, "lines.     \r",
                items = s.split()
                S1[items[0]] = [float(i) for i in items[2].split(";")]
                Ssum += sum(S1[items[0]][ntrim:-ntrim])
        print >> sys.stderr, "Prcessed ",i0, "lines.     "
        print >>sys.stderr, "Time used: %-3.2f secs.\n" % (time.clock()-starttime)

        # Read V1 file
        print >>sys.stderr, "Reading {0}".format(V)
        starttime=time.clock()
        Vsum = 0
        V1 = {}
        i0 = 0
        with open(V) as fh:
            for v in fh:
                i0 += 1
                if i0 % 100 == 0:
                    print >> sys.stderr, "Prcessed ",i0, "lines.     \r",
                items = v.split()
                V1[items[0]] = [float(i) for i in items[2].split(";")]
                Vsum += sum(V1[items[0]][ntrim:-ntrim])
        print >> sys.stderr, "Prcessed ",i0, "lines.     "
        print >>sys.stderr, "Time used: %-3.2f secs.\n" % (time.clock()-starttime)
        
        # Calculating normalization factors
        print >>sys.stderr, "Calculating normalization factors:"
        print >>sys.stderr, "Number of sequences:",min(len(V1),len(S1))
        print >> sys.stderr, "sum(S) = {0}, sum(V) = {1}".format(Ssum, Vsum)
        sk = (Ssum + Vsum)/2.0/Ssum
        vk = (Ssum + Vsum)/2.0/Vsum
        print >> sys.stderr, "Normalization ratio: S = %-3.2f, V = %-3.2f\n" % (sk, vk)

        # Read fasta file and generate PARS
        print >> sys.stderr, "Readling fasta file and generating PARS list ..."
        starttime=time.clock()
        PARSlst = []
        FastClst = []
        i0 = 0
        with open(fafile) as fh, open(prefix+".fp", 'w') as fhp, open(prefix+".fc", 'w') as fhc:
            for fa in IO.FastaReader(fafile):
                i0 += 1
                if i0 % 100 == 0:
                    print >> sys.stderr, "Prcessed ",i0, "items.     \r",
                if maxlen == 0 or len(fa.seq) <= maxlen:
                    tfp = PARS(fa.name,fa.seq,S1[fa.name],V1[fa.name])
                    # scipy.stats.fisher_exact and FDR
                    fdr_S = Algorithm.fdr_bh([stats.fisher_exact(x,alternative='greater')[1] for x in zip(zip(tfp.loops,Ssum-tfp.loops),zip(tfp.stems,Vsum-tfp.stems))])
                    fdr_V = Algorithm.fdr_bh([stats.fisher_exact(x,alternative='less')[1] for x in zip(zip(tfp.loops,Ssum-tfp.loops),zip(tfp.stems,Vsum-tfp.stems))])
                    # p value and fdr
                    # pv = [fisher.pvalue(*x) for x in zip(tfp.loops,Ssum-tfp.loops,tfp.stems,Vsum-tfp.stems)]
                    # fdr_S = Algorithm.fdr_bh([i.right_tail for i in pv])
                    # fdr_V = Algorithm.fdr_bh([i.left_tail  for i in pv])
                    # calculate constraints
                    cons = numpy.repeat(".",len(tfp))
                    for i in range(ntrim,len(cons)-ntrim): # flanking ntrim bases are not considered.
                        if fdr_S[i] < threshold:
                            cons[i] = "x"
                        elif fdr_V[i] < threshold:
                            cons[i] = '|'
                    # Normalization
                    tfp.normalize(sk, vk, ntrim)
                    PARSlst.append(tfp)
                    print >> fhp, tfp

                    # print constraints file
                    tfc = FastC(tfp.name,tfp.seq,cons.tostring())
                    FastClst.append(tfc)
                    print >> fhc, tfc

        print >> sys.stderr, "Prcessed ",i0, "items.     "
        print >> sys.stderr, "{0} sequences passed the maxlen filter.".format(len(PARSlst))
        print >>sys.stderr, "Time used: %-3.2f secs.\n" % (time.clock()-starttime)

        print >> sys.stderr, "Done."
        return (Ssum, Vsum, PARSlst, FastClst)
    readDataToPARS=staticmethod(readDataToPARS)
    def FastaReader(infile):
        '''Read sequence files.'''
        # Read lines
        with open(infile) as fh:
            line = fh.next()
            if line[0] != ">":
                raise ValueError("Records in Fasta files should start with '>' character")
            line = line.lstrip('>').split()
            name = line[0]
            seq = ''
            while True:
                try:
                    line = fh.next()
                except:
                    if seq != '':
                        yield Fasta(name, seq)
                    raise StopIteration
                if line[0] != ">":
                    seq += line.rstrip()
                else:
                    yield Fasta(name, seq)
                    line = line.lstrip('>').split()
                    name = line[0]
                    seq = ''
    FastaReader=staticmethod(FastaReader)
    def PARSReader(infile):
        ''' Read PARS file (sequences, S1 depth and V2 depth) and calculate the constraints with threshold. '''
        with open(infile) as fh:
            for line in fh:
                name = line.lstrip(">").rstrip()
                seq  = fh.next().rstrip()
                S    = numpy.array([float(i) for i in fh.next().split(";")])
                V    = numpy.array([float(i) for i in fh.next().split(";")])
                yield PARS(name, seq, S, V)
        raise StopIteration
    PARSReader=staticmethod(PARSReader)
    def FastCReader(infile):
        ''' Read FastC file. '''
        with open(infile) as fh:
            for line in fh:
                name = line.lstrip(">").rstrip()
                seq  = fh.next().rstrip()
                constraints = fh.next().rstrip()
                yield FastC(name, seq, constraints)
    FastCReader=staticmethod(FastCReader)
    def FastSReader(infile):
        ''' Read FastS file. '''
        with open(infile) as fh:
            name = ''
            seq = ''
            structures = []
            scores = []
            for line in fh:
                if line.startswith('>'):
                    if name != '':
                        yield FastS(name, seq, structures, scores)
                    name = line.lstrip('>').rstrip()
                    seq  = fh.next().rstrip()
                    structures = []
                    scores = []
                else:
                    lstr = line.split()
                    structures.append(lstr[0])
                    try:
                        scores.append(float(lstr[1].rstrip(")").lstrip("(")))
                    except:
                        scores.append(1.0)
            if name != '':
                yield FastS(name, seq, structures, scores)
        raise StopIteration
    FastSReader=staticmethod(FastSReader)

class Predictor(object):
    '''
    RNA structure predictor.
    Usage:
        structure, score = Predictor.RNAfold(tFastC, Temperature, threshold)
        print score, structure
        
    '''
    home = os.path.expanduser('~')
    def FoldMerge(tfastc, T = 37, predictors=['RNAfold','Fold','pknots','UNAFold','sfold']):
        ''' Merge the folded structures from multiple software. '''
        predictor_dict = {'RNAfold': Predictor.RNAfold, 'Fold': Predictor.Fold, 'pknots':  Predictor.pknots, 'mfold':  Predictor.mfold, 'UNAFold':  Predictor.UNAFold, 'sfold':Predictor.sfold,'pknotsRG':Predictor.pknotsRG, 'ipknot':Predictor.ipknot}
        structures = []
        scores = []
        for predictor in predictors:
            if predictor_dict.has_key(predictor):
                tfasts = predictor_dict[predictor](tfastc)
                structures.extend(tfasts.structures)
                scores.extend(tfasts.scores)
        # unique structure
        struct_dict= {}
        for st, sc in izip(structures,scores):
            if struct_dict.has_key(st):
                struct_dict[st] = min(sc, struct_dict[st]) # energy the lower the better
            else:
                struct_dict[st] = sc
        return FastS(tfastc.name, tfastc.seq, struct_dict.keys(), struct_dict.values())
    FoldMerge=staticmethod(FoldMerge)
    def RNAfold(tfastc, T = 37, threshold = 0):
        ''' Call RNAfold to predict the structure. '''
        # Run RNAfold with contraints
        p = Popen(['RNAfold','-C', '-T', str(T)], stdin = PIPE, stdout = PIPE, stderr = PIPE)
        p.stdin.write(str(tfastc))
        pstdout, pstderr = p.communicate()
        
        # Check the threshold here
        # to do 

        # Extract structure and score
        pre = re.compile(r'(\S+)\s+\((\s*-*\d+\.\d+)\)')
        s = pstdout.rstrip().split('\n')[-1]
        structure , score = pre.search(s).groups()
        if os.path.isfile(tfastc.name+"_ss.ps"):
            os.remove(tfastc.name+"_ss.ps")
        return FastS(tfastc.name, tfastc.seq, [structure], [float(score)])
    RNAfold=staticmethod(RNAfold)
    def Fold(tfastc, T = 37, threshold = 0):
        '''
        Call Fold (RNA Structure package) to predict structures.
        Note: Fold won't give structures if it cannot satisfy all the constraints. [Fix this in the future].
        '''
        # check DATAPATH environment variable
        if not os.environ.has_key('DATAPATH'):
            raise KeyError("Fold (RNA Structure) Error: Please set environment variable $DATAPATH to the location of the data_tables.")
        T = Utils.TempConverter(T,'C','K') # from C to K
        (fdfa, faname) = tempfile.mkstemp(suffix='.fa')
        (fdct, ctname) = tempfile.mkstemp(suffix='.ct')
        (fdcn, cnname) = tempfile.mkstemp(suffix='.CON')
        with os.fdopen(fdfa, 'w') as fh:
            fh.write(">{0}\n{1}\n".format(tfastc.name,tfastc.seq))
        with os.fdopen(fdcn, 'w') as fh:
            fh.write(Utils.PARSToFold(tfastc.constraints))
        os.close(fdct)
        structures = []
        scores = []
        try:
            # Run Fold
            # Fold 
            p = Popen(['Fold', faname, ctname, '-T', str(T), '-C', cnname], stdin = None, stdout = PIPE, stderr = PIPE)
            pstdout, pstderr = p.communicate()
            if pstderr:
                raise ValueError("Fold (RNA Structure package) run error: {0}".format(pstderr))
            # Parse the *.ct file
            tfs = Utils.ct2dot(ctname)
        finally:
            os.remove(faname)
            os.remove(ctname)
            os.remove(cnname)
        if len(structures) == 0:
            print >> sys.stderr, "No structures predicted by Fold."
        return tfs # FastS(tfastc.name, tfastc.seq, structures, scores)
    Fold=staticmethod(Fold)
    def pknots(tfastc, T = 37, threshold = 0):
        '''
        Fold by pknots. 
        pknots doesn't have temperature parameter.
        pknots has -k parameter for pseudoknots folding.
        '''
        if T != 37:
            print >> sys.stderr, "WARNING: pknots doesn't accept temperature != 37 !"
            return FastS(tfastc.name, tfastc.seq, [], [])
        (fdfa, faname) = tempfile.mkstemp(suffix='.fa')
        (fdct, ctname) = tempfile.mkstemp(suffix='.ct')
        with os.fdopen(fdfa, 'w') as fh:
            fh.write(">{0}\n{1}".format(tfastc.name,tfastc.seq))
        os.close(fdct)
        structures = []
        scores = []
        try:
            # Run pknots
            p = Popen(['pknots', '-k', faname, ctname], stdin = None, stdout = PIPE, stderr = PIPE)
            pstdout, pstderr = p.communicate()
            if pstderr:
                raise ValueError("ERROR: pknots run error: {0}".format(pstderr))
            # Parse the *.ct file
            #structures, scores = Utils.ct2dot(ctname)
            structures, scores = Utils.pknots2dot(ctname)
        finally:
            os.remove(faname)
            os.remove(ctname)
        if len(structures) == 0:
            print >> sys.stderr, "No structures predicted by pknots."
        return FastS(tfastc.name, tfastc.seq, structures, scores)
    pknots=staticmethod(pknots)
    def pknotsRG(tfastc, T = 37, threshold = 0):
        ''' Fold by pknotsRG. '''
        if T != 37:
            print >> sys.stderr, "WARNING: pknotsRG doesn't accept temperature != 37 !"
            return FastS(tfastc.name, tfastc.seq, [], [])
        structures = []
        scores = []
        try:
            p = Popen(['pknotsRG', '-s'], stdin = PIPE, stdout = PIPE, stderr = PIPE)
            p.stdin.write(">{0}\n{1}".format(tfastc.name, tfastc.seq))
            pstdout, pstderr = p.communicate()
        except OSError:
            raise OSError('pknotsRG is not installed.')
        if pstderr:
            raise IOError('ERROR: pknotsRG run error: '+pstderr)
        # parse the output
        for line in pstdout.split('\n')[6:-1]: # first 5 lines are descriptions
            structure, score = line.split()
            score = float(score[1:-2]) ## (-5.80)
            if score < 0 and structure not in structures:
                structures.append(structure)
                scores.append(score) 
        return FastS(tfastc.name, tfastc.seq, structures, scores)
    pknotsRG=staticmethod(pknotsRG)
    def mfold(tfastc, T = 37, threshold = 0):
        '''
        Fold by mfold.
        mfold SEQ=test.fa AUX=test.con T=37 RUN_TYPE=html
        '''
        (fdfa, faname) = tempfile.mkstemp(suffix='.fa')
        (fdcn, cnname) = tempfile.mkstemp(suffix='.con')
        with os.fdopen(fdfa, 'w') as fh: # with statement will close the fh and fd in the end
            fh.write(">{0}\n{1}\n".format(tfastc.name,tfastc.seq))
        with os.fdopen(fdcn, 'w') as fh:
            fh.write(Utils.PARSToMfold(tfastc.constraints))
        structures = []
        scores = []
        try:
            # mfold 
            p = Popen(['mfold', 'SEQ='+faname, 'AUX='+cnname, 'T={0}'.format(T), 'RUN_TYPE=html'], stdin = None, stdout = PIPE, stderr = PIPE) 
            pstdout, pstderr = p.communicate()
            if pstderr:
                raise ValueError("ERROR: mfold run error: {0}".format(pstderr))
        finally:
            os.remove(faname)
            os.remove(cnname)
            # Parse the *ct file
            idx = 1
            bname = os.path.basename(faname)
            fn = "{0}_{1}.ct".format(bname,idx)
            while os.path.isfile(fn):
                tfs = Utils.ct2dot(fn)
                structures.extend(tfs.structures)
                scores.extend(tfs.scores)
                idx += 1
                fn = "{0}_{1}.ct".format(bname,idx)
            for fn in os.listdir("."):
                if fn.startswith(bname):
                    os.remove(fn)
        if len(structures) == 0:
            print >> sys.stderr, "No structures predicted by mfold."
        if os.path.isfile('date.test'):
            os.remove('date.test')
        return FastS(tfastc.name, tfastc.seq, structures, scores) 
    mfold=staticmethod(mfold)
    def UNAFold(tfastc, T = 37, threshold = 0):
        ''' Fold by UNAFold. '''
        # Create temp file for fa and constraints
        (fdfa, faname) = tempfile.mkstemp(suffix='.fa',dir=".")
        (fdcn, cnname) = tempfile.mkstemp(suffix='.aux',dir=".")
        faname = os.path.basename(faname)
        cnname = os.path.basename(cnname)
        with os.fdopen(fdfa, 'w') as fh:
            fh.write(">{0}\n{1}\n".format(tfastc.name,tfastc.seq))
        with open(cnname, 'w') as fh:
            fh.write(Utils.PARSToMfold(tfastc.constraints))
        structures = []
        scores = []
        try:
            # mfold 
            p = Popen(['UNAFold.pl', '-t', str(T), '-c', cnname, '--run-type=html', faname], stdin = None, stdout = PIPE, stderr = PIPE) 
            pstdout, pstderr = p.communicate()
            if pstderr:
                print >> sys.stderr, "ERROR: UNAFold run error: {0}".format(pstderr)
        finally:
            os.remove(faname)
            os.remove(cnname)
            # Parse the *ct file
            bname = os.path.basename(faname)
            for fn in os.listdir("."):
                if fn.startswith(bname):
                    if fn.endswith(".ct"):
                        tfs = Utils.ct2dot(fn)
                        for st,sc in izip(tfs.structures,tfs.scores):
                            if st not in structures:
                                structures.append(st)
                                scores.append(sc)
                    os.remove(fn)
        if len(structures) == 0:
            print >> sys.stderr, "No structures predicted by UNAFold."
        return FastS(tfastc.name, tfastc.seq, structures, scores) 
    UNAFold=staticmethod(UNAFold)
    def ipknot(tfastc, T = 37, threshold = 0):
        ''' ipknot fold. '''
        if T != 37:
            print >> sys.stderr, "WARNING: pknotsRG doesn't accept temperature != 37 !"
            return FastS(tfastc.name, tfastc.seq, [], [])
        (fdfa, faname) = tempfile(suffix='.fa')
        with os.fdopen(fdfa) as fh:
            fh.write(">{0}\n{1}".format(tfastc.name, tfastc.seq))
        try:
            p = Popen(['ipknot', '-r', '10', faname], stdout = PIPE, stderr = PIPE)
            pstdout, pstderr = p.communicate()
        except OSError:
            raise OSError('ERROR: ipknot is not installed.')
        finally:
            os.remove(faname)
        # Parse the result
        if pstderr:
            raise IOError('ERROR: ipknot run error: '+pstderr)
        structure =  pstdout.split('\n')[-2].split()
        if structure.strip(".") != "":
            return FastS(tfastc.name, tfastc.seq, [structure], [0.0])
        return FastS(tfastc.name, tfastc.seq, [], [])
    ipknot=staticmethod(ipknot)
    def sfold(tfastc, T = 37, threshold = 0):
        ''' Fold by UNAFold. '''
        # Create temp file for fa and constraints
        prefix = tempfile.mkdtemp(dir=".")
        (fdfa, faname) = tempfile.mkstemp(suffix='.fa',dir='.')
        (fdcn, cnname) = tempfile.mkstemp(suffix='.aux',dir='.')
        with os.fdopen(fdfa, 'w') as fh:
            fh.write(">{0}\n{1}\n".format(tfastc.name,tfastc.seq))
        # constraints
        cns = False
        if tfastc.constraints.find('|') != -1 or tfastc.constraints.find('x') != -1:
            cns = True
            with os.fdopen(fdcn, 'w') as fh:
                fh.write(Utils.PARSToMfold(tfastc.constraints))
        structures = []
        scores = []
        try:
            # sfold
            if cns:
                p = Popen([Predictor.home+'/software/RNAStructure/sfold-2.2/bin/sfold', '-f', cnname, '-o', prefix, faname], stdin = None, stdout = PIPE, stderr = PIPE)
            else:
                p = Popen([Predictor.home+'/software/RNAStructure/sfold-2.2/bin/sfold', '-o', prefix, faname], stdin = None, stdout = PIPE, stderr = PIPE)
            pstdout, pstderr = p.communicate()
            if pstderr:
                print >> sys.stderr, "ERROR: sfold run error: {0}".format(pstderr)
            if os.path.isdir(prefix+"/clusters"): # clusters exists
                for f in os.listdir(prefix+"/clusters"):
                    if f.endswith(".ct"):
                        tfs = Utils.ct2dot(prefix+"/clusters/"+f)
                        for st,sc in izip(tfs.structures,tfs.scores):
                            structures.append(st)
                            scores.append(sc)
        finally:
            os.remove(faname)
            os.remove(cnname)
            os.system('rm -rf '+prefix)
        return FastS(tfastc.name,tfastc.seq, structures, scores)
    sfold=staticmethod(sfold)
    def snoGPS(tfa, T = 37, threshold =0):
        '''
        Predict H/ACA box snoRNAs.
        '''
        pass
        return tfs
    snoGPS=staticmethod(snoGPS)

class Algorithm(object):
    ''' Algorithm for RNA structure prediction. '''
    def Greedy(tfastpars, tfasts, sthreshold = 2, vthreshold = -2):
        '''
        Find the structures at match the experiment data.
        Example:
            loops       = 1  3  3  0  1  6  6  0  4  4  0  3  3  0  1  6  6  0  1  
            stems       = 0  1  1  3  3  1  0  3  0  1  3  0  1  6  6  0  1  1  0 
            cons        = .  x  x  |  |  x  x  |  x  x  |  x  x  |  |  x  x  .  .
            struct      = .  .  .  (  (  (  .  (  .  .  )  .  )  )  )  .  .  .  . 
            loop score       3  3           6     4  4     3           6  6  
            stem score             3  3        3        3        6  6 
        The score for this strtucture will be sum of loop and stem depth that satisfy the constraints.
        Then we remove the depth compatible to the best structure by taking their median depth out.
        loops = loops - 4 and stems = stems - 3
            loops       = 1  0  0  0  1  6  2  0  0  0  0  0  3  0  1  2  2  0  1  
            stems       = 0  1  1  0  0  1  0  0  0  1  0  0  1  3  3  0  1  1  0 
            cons        = .  .  .  |  |  x  x  |  x  x  |  x  x  |  |  x  x  .  .
            struct      = .  (  (  .  (  .  .  (  .  .  )  .  )  )  )  .  .  .  . 
            loop score       3  3           6     5  4     3           6  6  
            stem score             3  3        3        3        6  6 

        '''
        # single structure
        if len(tfasts.structures) ==1:
            return copy.deepcopy(tfasts)
        # multiple structures
        # data preparation
        loops = copy.deepcopy(tfastpars.loops)
        stems = copy.deepcopy(tfastpars.stems)
        tfastc = tfastpars.toFastC(sthreshold, vthreshold)
        cons = numpy.array(list(tfastc.constraints))
        structures = []
        for st in tfasts.structures:
            structures.append(numpy.array(list(st)))
        scores = copy.deepcopy(tfasts.scores)
        # start iteration
        matchstructures = []
        matchscores = []
        medianscore = 1000000
        while medianscore > 0 and len(structures) > 0:

            # find best match
            bestscore = 0
            bestidx = 0
            bestenergy = scores[0]
            for i,st in enumerate(structures):
                lscore = numpy.median(loops[numpy.where(st == '.')])
                sscore = numpy.median(stems[numpy.where(st != '.')])
                score = max(lscore, sscore)
                if score > bestscore or (score == bestscore and scores[i] < bestenergy): # best score or (equal score and best energy)
                    bestidx = i
                    bestscore = score
                    bestenergy = scores[i]
            medianscore = bestscore
            print bestscore
            # remove the compatible depth [ to be modified in the future.]
            loops[numpy.where(structures[i] == '.')] -= numpy.median(loops[numpy.where(structures[i] == '.')])
            loops[loops<0] = 0
            stems[numpy.where(structures[i] != '.')] -= numpy.median(stems[numpy.where(structures[i] != '.')])
            stems[stems<0] = 0
            # record the best structure in the matchstructures
            matchstructures.append(structures[i].tostring())
            matchscores.append(scores[i])
            # remove the best structure from structures.
            del structures[i]
            del scores[i]
        return FastS(tfasts.name,tfasts.seq, matchstructures, matchscores)
    Greedy=staticmethod(Greedy)
    def simulation(fs,numR,nS=1,nV=1):
        '''
        Simulate reads for structures with a given percentages.
            fs: FastS object contains multiple structures and an arbitrary proprotion.
            numR: number of RNA molecular
            nS, nV: proportion of reads for S and V.
        Note:
            numR is attributed to loops and stems by nS:nV.
            nS and nV are attributed to structures by scores of structures.
        '''        
        # initiation
        L = len(fs)
        M = len(fs.structures)
        S = numpy.zeros(L)
        V = numpy.zeros(L)
        # ranges of each structure
        scores = numpy.array(fs.scores,dtype=float)
        # Number of RNA molecular for S and V
        nS = round(1.0*numR*nS/(nS+nV))
        nV = numR - nS
       
        # z matrix
        z = numpy.zeros((L,M))
        for k in range(L):
            for j in range(M):
                if fs.structures[j][k] != ".": # k is stem in j
                    z[k,j] = 1
                    
        # generate random reads
        numpy.random.seed(int(time.time()*100)%100)
        for j in xrange(M):
            # nuber of RNA molecular for each structure
            # loops
            idx = numpy.nonzero(1-z[:,j])[0]
            l = len(idx)
            nSj = round(nS*scores[j]/sum(scores))*l
            for i in numpy.random.randint(l,size=nSj):
                S[idx[i]] += 1
            # stems
            idx = numpy.nonzero(z[:,j])[0]
            l = len(idx)
            nVj = round(nV*scores[j]/sum(scores))*l
            for i in numpy.random.randint(l,size=nVj):
                V[idx[i]] += 1
        return PARS(fs.name,fs.seq,S,V)
    simulation=staticmethod(simulation)
    def EM(fp,fs,centroids=None,threshold = 1e-6, maxiter=100):
        ''' 
        EM algorithm to calculate a percentage for each structure centroid. 
        '''
        def normalize(x,p):
            s = sum(x*p)
            if s == 0:
                return x
            else:
                return x*p/s
        
        if centroids is None:
            centroids = range(len(fs.structures))
        M = len(centroids) # Number of clusters
        if M ==0:
            return [0],[0,0,0,0]
        L = len(fp.loops)  # Length of RNA
        
        S = numpy.zeros(L)
        V = numpy.zeros(L)
        S += fp.loops
        V += fp.stems
       
        # Calculate Zkj
        z = numpy.zeros((L,M))
        for k in range(L):
            for j in range(M):
                if fs.structures[centroids[j]][k] != ".": # k is stem in j
                    z[k,j] = 1
        # Initial percentage
        pi = numpy.zeros(M)+1.0/M
        zV = numpy.apply_along_axis(sum,0,z) # number of stems in each structure
        zS = L-zV # number of loops in each structure
        
        # iteration
        cnt = 0
        smS = 0
        smV = 0
        while (True):
            # E step: E[P(X,Z|theta)]
            # normalize the possibilities by row
            ES = numpy.apply_along_axis(lambda x:normalize(x,pi),1,1-z)
            EV = numpy.apply_along_axis(lambda x:normalize(x,pi),1,z)
            # expected reads for each structure
            muS = numpy.zeros(M)
            muV = numpy.zeros(M)
            for k in xrange(L):
                for j in xrange(M):
                    muS[j] += S[k]*ES[k,j]
                    muV[j] += V[k]*EV[k,j]
            smS = sum(muS)
            smV = sum(muV)
            mu = muS/zS + muV/zV
            # M step: argmax(E[P(X,Z|theta)])
            mu /= sum(mu)
            
            # check if converge
            if cnt > maxiter or numpy.sum(numpy.abs(mu-pi)) < threshold:
                pi = mu
                break
            pi = mu
            cnt += 1
        print >> sys.stderr, "EM iterates {0} times.".format(cnt)
        return pi, [smS,sum(S),smV,sum(V)]
    EM=staticmethod(EM)
    def fitness(fp,fs,pi=[1]):
        '''
        Calculate fitness, the percentage of compatible reads, given all reads, structures and percentages: 
        Input: 
            fp: normalized PARS object. Flanking bases and outliers are trimmed.
            fs: FastS object with known or predicted structures.
            pi: percentage of each structures. sum(pi)=1.
        Output:
            smS: number of compatible S reads.
            S:   total number of S reads.
            smV: number of compatible V reads.
            V:   total number of V reads.
        '''
        def normalize(x,p):
            ''' Normalize conditional probabilities. '''
            s = sum(x*p)
            if s == 0:
                return x
            else:
                return x*p/s
        M = len(fs.structures)
        L = len(fp)
        if M == 0:
            return [0,0,0,0]
        S = numpy.zeros(L)
        V = numpy.zeros(L)
        S += fp.loops
        V += fp.stems
        # Calculate Zij
        z = numpy.zeros((L,M))
        for i in range(L):
            for j in range(M):
                if fs.structures[j][i] != ".": # k is stem in j
                    z[i,j] = 1
        # Expected depth
        ES = numpy.apply_along_axis(lambda x:normalize(x,pi),1,1-z)
        EV = numpy.apply_along_axis(lambda x:normalize(x,pi),1,z)
        # expected reads for each structure
        muS = numpy.zeros(M)
        muV = numpy.zeros(M)
        for i in xrange(L):
            for j in xrange(M):
                muS[j] += S[i]*ES[i,j]
                muV[j] += V[i]*EV[i,j]
        smS = sum(muS)
        smV = sum(muV)
        return smS,sum(S),smV,sum(V)
    fitness=staticmethod(fitness)
    def spectralClustering(data):
        '''
        Do spectral clustering on symmetric distance matrix, data, into K groups, in which K is determined by eigengap heuristic method [1].
        Usage:
            centroids, clusters = spectralClustering(data,k)
        Example:
            data = [[ 0.  4.  4.  2.  2.]
                    [ 4.  0.  4.  4.  4.]
                    [ 4.  4.  0.  4.  4.]
                    [ 2.  4.  4.  0.  4.]
                    [ 2.  4.  4.  4.  0.]]
            k = 2
            centroid_idx, clusters = spectralClustering(data)
            Output:
                > print centroids
                [0,1]
                > print clusters
                [array([0, 3, 4]), array([1, 2])]
        References:
            [1] von Luxburg, U. (2007). "A tutorial on spectral clustering." Statistics and Computing 17(4): 395-416.
        '''
        n = data.shape[0]
        sd = numpy.std(data)
        
        # Affinity matrix
        A = numpy.zeros(data.shape,dtype=float)
        for i in range(n):
            for j in range(i):
                A[i,j] = numpy.exp(-sum((data[i,:]-data[j,:])**2/2/sd**2))
                A[j,i] = A[i,j]
                
        # Diagonal matrix
        D = numpy.zeros(data.shape)
        for i in range(n):
            D[i,i] = sum([A[i,j] for j in range(n)])
            
        # Scaled matrix L from D
        S = numpy.linalg.inv(D)**0.5
        L = numpy.dot(S,A)
        L = numpy.dot(L,S)
        # Eigen values and vectors: w, v
        w, v = numpy.linalg.eig(L)
        
        # find the best K using eigengap heuristic method 
        # von Luxburg, U. (2007). "A tutorial on spectral clustering." Statistics and Computing 17(4): 395-416.
        sw = w.argsort()
        eigengaps = [w[sw[i+1]]-w[sw[i]] for i in range(n-1)]
        k = numpy.argmax(eigengaps) + 1
        # plot(range(n-1),eigengaps,'*')
        
        # get the eigen vectors for the first smallest eigen values    
        X = v[:,sw[:k]]

        # normalize X
        Y = numpy.apply_along_axis(lambda x: x/numpy.sqrt(sum(x**2)),1,X)
        
        # do k-means clustering
        km = KMeans(n_clusters=k, init='k-means++', max_iter=100, n_init=5, verbose=False)
        rst = km.fit(Y)
        centroids = rst.cluster_centers_
        labels = rst.labels_
        
        # index of centroids
        centroid_idx = numpy.zeros(k,dtype=numpy.int32)
        for i,centroid in enumerate(centroids):
            cur = 0
            curdis = -1
            for j,point in enumerate(Y):
                dis = numpy.sum((centroid-point)**2)
                if dis < curdis or curdis == -1:
                    cur = j
                    curdis = dis
            centroid_idx[i]=cur
        return centroid_idx,labels
    spectralClustering=staticmethod(spectralClustering)    
    def kmeans(data, k, maxiter = 20 ): # obsoleted
        ''' K-means clustering. '''
        n = data.shape[0] 
        labels = numpy.repeat(0,n)
        minima = None # global minima
        for i in xrange(maxiter):
            # generate k centers randomly
            centroids = random.sample(range(n),k) # current k centers
            if debug: print "Start K-means: k =", k, ", # of samples:", n
            while True:
                # get labels for each elements    
                for j in range(n):
                    mindis = data[j,centroids[0]] # set label to the first centroid
                    labels [j] = centroids[0]
                    for i in range(k): # find the nearest centroid
                        if data[j,centroids[i]] < mindis:
                            labels[j] = centroids[i]
                            mindis = data[j,centroids[i]] # the nearest centroid
                if debug: print "centroids", centroids, "labels",labels
                # Find new centroids
                precentroids = [i for i in centroids]
                distance_square_sum = numpy.repeat(10000000,k) # distance square sum 
                # For each point, check if it is the centroid of its cluster.
                for i in range(n):
                    sumi = 0. # sum of square distance
                    cnt = 0 # count of samples
                    for j in range(n):
                        if labels[i] == labels[j]: 
                            sumi += data[i,j]**2
                            cnt  +=1
                    sumi /= cnt
                    idx = precentroids.index(labels[i]) # index of the centroid/ksum 
                    if sumi < distance_square_sum[idx]: # possible new center
                        distance_square_sum[idx] = sumi
                        centroids[idx] = i
                if debug: print "Total score:",sum(distance_square_sum)
                if numpy.array_equal(centroids,precentroids): # check if converge
                    break
            # Check if minima.
            tsum = sum(distance_square_sum)
            if minima is None or tsum < minima:
                minima = tsum
                bestcluster = (sorted(centroids),numpy.array(labels),tsum)
        return bestcluster
    kmeans=staticmethod(kmeans)
    def BIC(data, centroids, labels, seqlen): # not working for unknown reason
        ''' BIC algorithm to find the best k of K-means. '''
        n = len(labels)
        k = len(centroids)
        R = len(labels)
        M = seqlen # M should be the dimension of the points.
        Ri = [len(labels[labels == centroids[i]]) for i in range(k)]
        if R == k:
            sigma = 0
        else:
            sigma = sum([data[i,labels[i]]**2 for i in xrange(R)])/float(R-k)
        lD = -0.5*numpy.log(2*numpy.pi) - 0.5*R*M*sigma - 0.5*R + 0.5*k**2 + sum([r*numpy.log(r) for r in Ri]) - R*numpy.log(R)
        return lD - 0.5*(M+1)*k*numpy.log(R)
    BIC=staticmethod(BIC)
    def hcluster(ndata,threshold=20):
        ''' Hierarchical clustering. '''
        data = numpy.array(ndata)
        R = data.shape[0]
        clusters = {i:[i] for i in range(R)} # cluster initiation.
        for k in range(R-1):
            # Find the pair with the shortest distance
            idx = numpy.where(data == numpy.min(data[data!=0]))
            i = idx[0][0]
            j = idx[1][0]
            if debug: print i,j,data[i,j]
            if data[i,j] <= threshold:                
                # Update data
                if debug: print data[i,:],len(clusters[i]),len(clusters[j])
                data[i,:] = (data[i,:]*len(clusters[i]) + data[j,:]*len(clusters[j]))/(len(clusters[i])+len(clusters[j]))
                data[:,i] = data[i,:]
                clusters[i].extend(clusters[j])
                del clusters[j]
                data[i,i] = 0
                data[j,:] = 0
                data[:,j] = 0
                if debug: print data
            else: # No pairs have distance < threshold
                break
        return clusters.values()
    hcluster=staticmethod(hcluster)    
    def fdr_bh(pvalues):
        n = len(pvalues)
        if n == 0:
            return []
        # sort pvlaues
        args, pv = zip(*sorted(enumerate(pvalues),key= lambda x:x[1]))
        bh_values = [0] * n
        pre_v = 0
        for i, pv in enumerate(pv):
            cur_v = min(1, pv * n /(i+1)) # correction
            pre_v = max(cur_v, pre_v)
            bh_values[args[i]] = pre_v
        return bh_values
    fdr_bh=staticmethod(fdr_bh)
    def rank(x):
        ''' rank of a list. '''
        array=numpy.array(x)
        temp = array.argsort()
        ranks = numpy.arange(len(array))[temp.argsort()]
        return ranks
    rank=staticmethod(rank)
    def trimed(x, lowbound = 0.05, upbound = 0.95):
        ''' Trim the top ones and the bottom ones, and set them to bound values. '''
        r = Algorithm.rank(x) # type(r) is numpy.ndarray
        l = len(r) - 1
        ub = x[numpy.where(r == round(upbound * l))[0]]
        lb = x[numpy.where(r == round(lowbound *l))[0]]
        if type(x) == numpy.ndarray:
            nx = numpy.array(x)
            nx[nx>ub] = ub
            nx[nx<lb] = lb
        else:
            nx = [ ub if i > ub else i for i in x]
            nx = [ lb if i < lb else i for i in nx]
        return nx
    trimed=staticmethod(trimed)
    def distanceMatrix(fs,method="rbp"):
        '''
        Calculate the distance matrix, given a FastS object with multiple structures.
        Options:
            method: 'top' means calculate the topological distance using wRNA.distance from Vienna RNA package [1].
                    'rbp'  means calculate the relaxed base-pair distance [2].
        References:
            [1] Hofacker, I. L., et al. (1994). "Fast Folding and Comparison of Rna Secondary Structures." Monatshefte Fur Chemie 125(2): 167-188.
            [2] Agius, P., et al. (2010). "Comparing RNA secondary structures using a relaxed base-pair score." RNA 16(5): 865-878.
        '''
        if method in ['top','rbp']:
            dist = method == 'top' and wRNA.distance or Algorithm.BP_distance
        else:
            dist = Algorithm.BP_distance
        if len(fs.structures)==1:
            return numpy.matrix([0])
        dim = len(fs.structures)
        dismatrix = numpy.array([0]*dim*dim, dtype=numpy.float)
        dismatrix.shape = (dim, dim)
        for i in xrange(dim):
            dismatrix[i,i] = 0
            for j in xrange(i):
                dismatrix[i,j] = dist(fs.structures[i], fs.structures[j])
                dismatrix[j,i] = dismatrix[i,j]
        return dismatrix
    distanceMatrix=staticmethod(distanceMatrix)
    def BP_distance(S1,S2, t = 1):
        '''
        Relaxed base-pair distance between two structures [1].
        Options:
            t: relaxed parameter.
        References:
            [1] Agius, P., et al. (2010). "Comparing RNA secondary structures using a relaxed base-pair score." RNA 16(5): 865-878.
        '''
        # basepairs for S1
        bps1 = []
        m = re.search('[\(\)]+',S1)
        pos= 0 
        while m is not None :            
            bps1.append((pos+m.start()+1,pos+m.end()))
            pos+=m.end()
            m = re.search('[\(\)]+',S1[pos:])
        # basepairs for S2
        bps2 = []
        m = re.search('[\(\)]+',S2)
        pos= 0 
        while m is not None :            
            bps2.append((pos+m.start()+1,pos+m.end()))
            pos+=m.end()
            m = re.search('[\(\)]+',S2[pos:])
        # pairwise distances
        ds = []
        for i,j in bps1:
            ds.append( min([max(abs(i-i1),abs(j-j1)) for i1,j1 in bps2]))
        for i,j in bps2:
            ds.append( min([max(abs(i-i1),abs(j-j1)) for i1,j1 in bps1]))
        # sort ds
        ds = numpy.array(sorted(ds,reverse=True))
        # calcluate min m
        minm = len(ds)
        for m in range(len(ds)+1):
            if numpy.all(ds[m:] <= t*m):
                minm = m
                break
        return minm
    BP_distance=staticmethod(BP_distance)

class Utils(object):
    ''' Utils for RNA structure prediction. '''
    DegreeAlphabet = ['F','K','C', 'R']
    AnyToC = { 'F': lambda x: (x-32.)/1.8, 'K': lambda x: x-273.16, 'R': lambda x: x*1.25 }
    CToAny = { 'F': lambda x: x*1.8+32., 'K': lambda x: x+273.16, 'R': lambda x: x* 0.8 }
    def TempConverter(temp, _from, _to):
        '''
        Convert temperature among different systems.
            C = Celsius
            F = Fahrenheit
            K = Kelvin (absolute temperature)
            R = Reaumur Equals
        '''
        _from = _from.upper()
        _to = _to.upper()
        if _from not in Utils.DegreeAlphabet or _to not in Utils.DegreeAlphabet:
            raise ValueError("Temperature format not supported. Please choose from ['F','C','K', 'R'].")
        if _from == 'C':
            return Utils.CToAny[_to](temp)
        if _to == 'C':
            return Utils.AnyToC[_from](temp)
        t = Utils.AnyToC[_from](temp)
        return Utils.CToAny[_to](t)
    TempConverter=staticmethod(TempConverter)
    def PARSToMfold(constraints):
        ''' Convert PARS constraints to mfold constraints. '''
        mconstraints = []
        for match in re.finditer(r'(\|+)', constraints):
            mconstraints.append('F {0} 0 {1}'.format(match.start()+1, match.end() - match.start()))
        for match in re.finditer(r'(x+)', constraints):
            mconstraints.append('P {0} 0 {1}'.format(match.start()+1, match.end() - match.start()))
        return '\n'.join(mconstraints)
    PARSToMfold=staticmethod(PARSToMfold)
    def PARSToFold(constraints):
        '''
        Convert PARS constraints to Fold (RNAStructure package) constraints.
        Example of Fold contraints:
            DS:
            2
            3
            -1
            SS:
            5
            7
            -1
        '''
        fconstraints = "DS:"
        for match in re.finditer(r'(\|+)', constraints):
            for i in range(match.start(),match.end()):
                fconstraints += "\n{0}".format(i+1)
        fconstraints += "\n-1\nSS:"
        for match in re.finditer(r'(x+)', constraints):
            for i in range(match.start(),match.end()):
                fconstraints += "\n{0}".format(i+1)
        fconstraints +="\n-1\nMod:\n-1\nPairs:\n-1 -1\nFMN:\n-1\nForbids:\n-1 -1"
        return fconstraints
    PARSToFold=staticmethod(PARSToFold)
    def ct2dot(ctname):
        ''' 
        CT format to DOT format.
        Input can be CT file names or a CT file string.
            50      dG = -6.2       seqname
            1       A       0       2       0       1       0       0
            2       U       1       3       0       2       0       0
            3       G       2       4       0       3       0       0
            4       A       3       5       0       4       0       0
            5       C       4       6       0       5       0       0
            6       A       5       7       0       6       0       0
            7       C       6       8       0       7       0       8
            8       A       7       9       36      8       7       9
            9       G       8       10      35      9       8       10
            10      C       9       11      34      10      9       11
            11      U       10      12      33      11      10      12
            12      U       11      13      31      12      11      13
            13      C       12      14      30      13      12      14
            14      A       13      15      29      14      13      15
        Note:
            (1) dG can be ENERGY, energy or dG.
            (2) the first 6 fields are required.
            (3) for RNA without structure predicted, no dG field provided.
        For pseudoknots like this:
            '(((..[[[...)))...]]]'
            The first seen pairs are labelled with '()', and the following conflict ones are labelled as '[]'
        '''
        structures = []
        scores = []
        try: # file name
            fh = open(ctname)
            cts = fh.readlines()
            fh.close()
        except: # string of CTs
            cts = ctname.split('\n')
        A = []
        B = []
        score = 0.
        seq = ''
        name = ''
        # suppose there might be multiple structures in the ct file.
        idx = 0
        while idx < len(cts):
            line = cts[idx].split()
            if len(line)>=6 and line[0] == line[5]:
                if line[5] == '1': # first line
                    if len(A) > 0:
                        s = Utils._ABToDot(A,B)
                        if s.find('(') != -1 or s.find('[') != -1:
                            structures.append(s)
                            scores.append(score)
                    A = [int(line[0])]
                    B = [int(line[4])]
                    seq += line[1]
                    # Parse header                
                    name = cts[idx-1].split()[-1]
                    if name.strip('.') == "":
                        name = "NONAME"
                    m = re.findall('-*\d+\.*\d*',cts[idx-1])
                    if len(m) >= 1:
                        ctlen = int(m[0])
                    if len(m) >=2:
                        score = float(m[1])
                else:
                    A.append(int(line[0]))
                    B.append(int(line[4]))
                    seq += line[1]
            idx +=1
        if len(A) > 0:
            s = Utils._ABToDot(A,B)
            if s.find('(') != -1 or s.find('[') != -1:
                structures.append(s)
                scores.append(score)
        return FastS(name,seq,structures,scores) 
    ct2dot=staticmethod(ct2dot)
    def _ABToDot(A, B):
        ''' Generate DOT structure from a list of pairs. '''
        structure = numpy.repeat('.', len(A))
        bs = [len(A) + 1]
        for a, b in izip(A, B):
            if a > b:
                try:
                    bs.remove(a)
                except:
                    pass
            elif b > min(bs):
                structure[a-1] = '['
                structure[b-1] = ']'
            else:
                structure[a-1] = '('
                structure[b-1] = ')'
                bs.append(b)
        return structure.tostring()
    _ABToDot=staticmethod(_ABToDot)
    def pknots2dot(pkname):
        ''' Convert pknots output to dot format. '''
        structures = []
        scores = []
        try:
            with open(pkname) as fh:
                pks = fh.readlines()
        except:
            pks = pkname.split('\n')
        idx = 0
        while idx < len(pks):
            # Find SEQ
            while idx < len(pks) and not pks[idx].startswith('SEQ'):
                idx += 1
            if idx >= len(pks):
                break
            # Read pairs
            A = []
            B = []
            idx += 1
            while not pks[idx].startswith('---'):
                A.extend([int(i) for i in pks[idx+1].split()])
                B.extend([int(i) for i in pks[idx+2].replace('.','0').split()])
                idx += 4
            # Parse pairs
            structures.append(Utils._ABToDot(A, B))
            # Find energy line
            while not pks[idx].startswith('energy'):
                idx += 1
            scores.append(float(pks[idx].split()[-1]))
        return (structures, scores)
    pknots2dot=staticmethod(pknots2dot)    
    def dot2ct(tfasts): # 
        ''' Convert DOT format to CT format. '''
        ctstring = []
        for st, sc  in izip(tfasts.structures, tfasts.scores):
            # print header
            ctstring.append ("%5d  ENERGY = %-3.2f  %s" % (len(tfasts), sc, tfasts.name))
            stack1=[]
            stack2=[]
            pairs={}
            for i,c in enumerate(st):
                if c == '(':
                    stack1.append(i+1)
                elif c == '[':
                    stack2.append(i+1)
                elif c == ')':
                    pairs[i+1] = stack1.pop()
                    pairs[pairs[i+1]] = i+1
                elif c == ']':
                    pairs[i+1] = stack2.pop()
                    pairs[pairs[i+1]] = i+1
            for i in xrange(1,len(tfasts)+1): # ###24#A######23###25####0###24
                ctstring.append( " %4d %s %7d%4d %4d %4d" % (i, tfasts.seq[i-1], i-1, i+1, pairs.get(i,0),i) )
        return '\n'.join(ctstring)
    dot2ct=staticmethod(dot2ct)
    def draw(fs, ftype = 'ps'):
        '''
        Draw structure in Postscript or SVG format.
        Input: FastS object
            >YIL140W
            AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU
            .(((.......)))............(((...)))...............    (-3.1)
            ..............(((.........(((...)))......)))......    (-3.0)
            ...........((((............))))...................    (-3.0)
        Usage:
            Utils.draw(fs, 'png')
        Output:
            YIL140W-1.png  YIL140W-2.png YIL140W-3.png
        '''
        ftype = ftype.lower()
        cts = Utils.dot2ct(fs)
        for i in range(len(fs.structures)):
            fprefix = "{0}-{1}".format(fs.name, i+1)
            if ftype == 'svg':
                wRNA.plot(cts,fprefix+".svg", 1, i+1)
            elif ftype == 'ps':
                wRNA.plot(cts,fprefix+".ps", 0, i+1)
            else:
                lps = wRNA.plot(cts,"", 0, i+1)
                try:
                    p = Popen(['convert', '-', fprefix+"."+ftype], stdin = PIPE, stdout = PIPE, stderr = PIPE)
                    p.stdin.write(lps)
                    pstdout, pstderr = p.communicate()
                except OSError as e:
                    raise OSError(e)
                finally:
                    if pstderr:
                        print >> sys.stderr, "ERROR: convert run error:",pstderr
        return
    draw=staticmethod(draw)

class Test(object):
    ''' Test Module. '''
    def testPredictor():
        ''' Test Predictor. '''
        # test FastC class
        T = 37
        threshold = 0
        tfastc = FastC('YIL140W','AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU','..xx......||..........xx...|......||..............')
        print "RNAfold (Vienna RNA package) Test:"
        print "Sequence for prediction:"
        print tfastc
        
        # test RNAfold
        print "\nRNAfold predicted structure:"
        print Predictor.RNAfold(tfastc, T, threshold)
        print 

        # test Fold
        print "\nFold (RNAStructure package) predicted structure:"
        print Predictor.Fold(tfastc,T, threshold)
        print 

        # test mfold
        print "\nmfold predicted structure:"
        print Predictor.mfold(tfastc,T, threshold)
        print 

        # test UNAfold
        print "\nUNAFold predicted structure:"
        print Predictor.UNAFold(tfastc, T, threshold)
        print 

        # test pknots
        print "\npknots predicted structure:"
        print Predictor.pknots(tfastc, T, threshold)
        print

        # test pknotsRG
        print "\npknotsRG predicted structure:"
        print Predictor.pknotsRG(tfastc, T, threshold)
        print
        
        # test ipknot
        print "\nipknot predicted structure:"
        print Predictor.ipknot(tfastc, T, threshold)
        print 

        # test FoldMerge
        print "\n Merge all the predicted structures together."
        print Predictor.FoldMerge(tfastc, T, predictors = ['RNAfold','pknots','Fold','mfold'])
        print
    testPredictor=staticmethod(testPredictor)
    def testUtils():
        ''' Test Utils. '''
        tfastc = FastC('YIL140W','AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU','.|xx.|.x..||.|...xx.x.xx...||.x|..||...|..........')
        tfasts = Predictor.RNAfold(tfastc, 37, 0)
        print "DOT format to CT format:"
        cts = Utils.dot2ct(tfasts)
        print cts
        print
        print "CT format back to DOT format:"
        print Utils.ct2dot(cts)
    testUtils=staticmethod(testUtils)

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    if len(sys.argv) == 1:
        sys.exit('''
        Module description:
            RNA Structure Prediction Module.
        Test:
            python wRNA.py test
        
        Usage:
            import wRNA
            tfastc = wRNA.FastC('YIL140W','AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU','.|xx.|.x..||.|...xx.x.xx...||.x|..||...|..........')
            print tfastc
            print wRNA.Predictor.RNAfold(tfastc)

        Output:
            >YIL140W
            AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU
            .|xx.|.x..||.|...xx.x.xx...||.x|..||...|..........
            >YIL140W
            AUGACACAGCUUCAGAUUUCAUUAUUGCUGACAGCUACUAUAUCACUACU
            ((...(..((((.((............)))..))))...)).........    (9.70)
        ''')
    else:
        Test.testPredictor()
        #Test.testUtils()




# Copyright (C) 2008 Simone Leo - CRS4. All Rights Reserved.
# 
# Permission to use, copy, modify, and distribute this software and its
# documentation for educational, research, and not-for-profit purposes, without
# fee and without a signed licensing agreement, is hereby granted, provided
# that the above copyright notice, this paragraph and the following two
# paragraphs appear in all copies, modifications, and distributions. Contact
# CRS4, Parco Scientifico e Tecnologico, Edificio 1, 09010 PULA (CA - Italy),
# +3907092501 for commercial licensing opportunities.
# 
# IN NO EVENT SHALL CRS4 BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,
# INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF
# THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF CRS4 HAS BEEN ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
# 
# CRS4 SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE. THE SOFTWARE AND ACCOMPANYING DOCUMENTATION, IF ANY, PROVIDED
# HEREUNDER IS PROVIDED "AS IS". CRS4 HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
# SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

"""
Functions for computing the False Discovery Rate (FDR) of a multiple test
procedure. The FDR is the expected proportion of falsely rejected hypotheses.

@see:
  Yoav Benjamini and Yosef Hochberg, I{Controlling the False Discovery Rate:
  A Practical and Powerful Approach to Multiple Testing}. Journal of the Royal
  Statistical Society, Series B (Methodological), Vol. 57, No. 1 (1995), pp.
  289-300.

  Yekutieli, D. and Benjamini, Y., I{Resampling-based false discovery rate
  controlling multiple test procedures for correlated test statistics}. J. of
  Statistical Planning and Inference, 82, pp. 171-196, 1999.

  John D. Storey (2002), I{A direct approach to false discovery rates}. Journal
  of the Royal Statistical Society: Series B (Statistical Methodology) 64 (3),
  479-498.
"""

__docformat__ = 'epytext en'


import operator
import random


def bh_rejected(pv, threshold):
    """
    Return the list of rejected p-values from C{pv} at FDR level C{threshold}.

    The downside of this approach is that the FDR level must be chosen in
    advance - the L{bh_qvalues} function yields a p-value-like output instead.

    @type pv: list
    @param pv: p-values from a multiple statistical test
    @type threshold: float
    @param threshold: the level at which FDR rate should be controlled

    @rtype: list
    @return: p-values of rejected null hypotheses
    """
    if threshold < 0 or threshold > 1:
        raise ValueError("the threshold must be between 0 and 1")
    if not pv:
        return []
    pv = sorted(pv)
    if pv[0] < 0 or pv[-1] > 1:
        raise ValueError("p-values must be between 0 and 1")
    m = len(pv)
    for i in xrange(m-1, -1, -1):
        if pv[i] <= float(i+1)*threshold/m:
            return pv[:i+1]
    return []


def bh_qvalues(pv):
    """
    Return Benjamini-Hochberg FDR q-values corresponding to p-values C{pv}.

    This function implements an algorithm equivalent to L{bh_rejected} but
    yields a list of 'adjusted p-values', allowing for rejection decisions
    based on any given threshold.

    @type pv: list
    @param pv: p-values from a multiple statistical test

    @rtype: list
    @return: adjusted p-values to be compared directly with the desired FDR
      level
    """
    if not pv:
        return []
    m = len(pv)
    args, pv = zip(*sorted(enumerate(pv), None, operator.itemgetter(1)))
    if pv[0] < 0 or pv[-1] > 1:
        raise ValueError("p-values must be between 0 and 1")
    qvalues = m * [0]
    mincoeff = pv[-1]
    qvalues[args[-1]] = mincoeff
    for j in xrange(m-2, -1, -1):
        coeff = m*pv[j]/float(j+1)
        if coeff < mincoeff:
            mincoeff = coeff
        qvalues[args[j]] = mincoeff
    return qvalues


def get_pi0(pv, lambdas):
    """
    Compute Storey's C{pi0} from p-values C{pv} and C{lambda}.

    this function is equivalent to::
    
        m = len(pv)
        return [sum(p >= l for p in pv)/((1.0-l) * m) for l in lambdas]
        
    but the above is C{O(m*n)}, while it needs only be C{O(m+n)
    (n = len(lambdas))}

    @type pv: list
    @param pv: B{SORTED} p-values vector
    @type lambdas: list
    @param lambdas: B{SORTED} lambda values vector

    @rtype: list
    @return: estimated proportion of null hypotheses C{pi0} for each lambda
    """
    m = len(pv)
    i = m - 1
    pi0 = []
    for l in reversed(lambdas):
        while i >= 0 and pv[i] >= l:
            i -= 1
        pi0.append((m-i-1)/((1.0-l)*m))
    pi0.reverse()
    return pi0


def storey_qvalues(pv, l=None):
    """
    Return Storey FDR q-values corresponding to p-values C{pv}.

    The main difference between B-H's and Storey's q-values is that the latter
    are weighted by the estimated proportion C{pi0} of true null hypotheses.

    @type pv: list
    @param pv: p-values from a multiple statistical test
    @type l: float
    @param l: lambda value for C{pi0} (fraction of null p-values) estimation

    @rtype: list
    @return: storey q-values corresponding to C{pv}
    """
    if not pv:
        return []
    m = len(pv)
    args, pv = zip(*sorted(enumerate(pv), None, operator.itemgetter(1)))
    if pv[0] < 0 or pv[-1] > 1:
        raise ValueError("p-values must be between 0 and 1")

    if l is None:
        # optimal lambda/pi0 estimation
        lambdas = [i/100.0 for i in xrange(0, 91, 5)]
        n = len(lambdas)
        pi0 = get_pi0(pv, lambdas)
        min_pi0 = min(pi0)
        mse = [0] * n        
        for i in xrange(1, 101):
            # compute bootstrap sample with replacement
            pv_boot = [pv[int(random.random()*m)] for j in xrange(m)]
            pi0_boot = get_pi0(sorted(pv_boot), lambdas)
            for j in xrange(n):
                mse[j] += (pi0_boot[j] - min_pi0) * (pi0_boot[j] - min_pi0)
        min_mse = min(mse)
        argmin_mse = [i for i, mse_i in enumerate(mse) if mse_i == min_mse]
        pi0 = min(pi0[i] for i in argmin_mse)
        pi0 = min(pi0, 1)
    else:
        try:
            l = float(l)
        except ValueError:
            raise TypeError("lambda must be a number")
        if l < 0 or l >= 1:
            raise ValueError("lambda must be within [0,1)")
        pi0 = get_pi0(pv, [l])
        pi0 = min(pi0[0], 1)

    qvalues = m * [0]
    mincoeff = pi0 * pv[-1]
    qvalues[args[-1]] = mincoeff
    for j in xrange(m-2, -1, -1):
        coeff = pi0*m*pv[j]/float(j+1)
        if coeff < mincoeff:
            mincoeff = coeff
        qvalues[args[j]] = mincoeff
    return qvalues



#!/usr/bin/python
#Last-modified: 28 Mar 2014 05:08:35 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <yfwang0405@gmail.com>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.1.0
# @author:  Yunfei Wang
# @contact: yfwang0405@gmail.com

# ------------------------------------
# python modules
# ------------------------------------

import os,sys
from setuptools import setup, find_packages, Extension

# ------------------------------------
# constants
# ------------------------------------

# ------------------------------------
# Misc functions
# ------------------------------------

# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__ == '__main__':
    if float(sys.version[:3])<2.6 or float(sys.version[:3])>=2.8:
        sys.stderr.write("CRITICAL: Python version must be 2.6 or 2.7!\n")
        sys.exit(1)

    # includepy = "%s/include/python%s" % (sys.prefix, sys.version[:3])
    with open("README",'r') as fh:
        long_description = fh.read()
    # ngslib version
    with open('VERSION','r') as fh:
        PROG, VERSION = fh.read().split()

    # Compile Kent lib
    if 'clean' in sys.argv:
        print >>sys.stderr, "Clean KentLib ..."
        os.system('cd external/KentLib && make clean && cd ../..')
        print >>sys.stderr, "Clean dist and egg info ..."
        os.system('if [ -d dist ]; then rm -rf dist; fi')
        os.system('if [ -f ngslib.egg-info ]; then rm ngslib.egg-info; fi')
        os.system('if [ -d ngslib.egg-info ]; then rm -rf ngslib.egg-info; fi')
    else:
        print >>sys.stderr, "Compile KentLib ..."
        os.system('cd external/KentLib && make && cd ../..')
    
    # install requirement
    install_requires = [ ["python>=2.6"],
                         ["numpy >= 1.4.1"],
                         ["pysam >= 0.6"]]
    # Python 2.6 requires argparse
    if float(sys.version[:3]) == 2.6:
        install_requires.append(["argparse >= 1.2.1"])
    
    setup(name=PROG,
          version=VERSION,
          author='Yunfei Wang',
          author_email='yfwang0405@gmail.com',
          url='http://tsznxx.appspot.com',
          license="GNU General Public License (GPL)",
          keywords = "Python Sequencing Bed BigWig TwoBit",
          description = ("Python Modules for Next-Generation Sequencing Data Analysis."),
          long_description = long_description,
          package_dir={'ngslib':'src'},
          packages = ['ngslib'],
          scripts=['bin/wBamToWig.py',
                   'bin/wBedToWig.py',    
                   'bin/wXLSToTXT.py',
                   'scripts/wBedExtend.py',
                   'scripts/wBedToFasta.py',
                   'bin/wSam2Bed.py',
                   'bin/wWigPlot.py',
                   'scripts/wFindNearestAnnotation.py',
                   'scripts/wGetTSS.py',
                   'scripts/wlncRNA.py',
                   'scripts/wGTFToTab.py',
                   'scripts/wTabToBed.py',
                   'scripts/wFormatFasta.py',
                   'scripts/wGCContent.py',
                   'scripts/wGetSeqByCoordinates.py',
                   'scripts/wGetSeqByName.py',
                   'scripts/wRandomBed.py',
                   'scripts/wFindNearestTwoAnnotation.py',
                   'scripts/wFindNearestAnnotation.py',
                   'scripts/wBedAnnotation.py',
                   'scripts/wBedGetWig.py',
                   'scripts/wGetIntron.py',
                   'scripts/wGetExon.py',
                   'scripts/wMergeSort.py'],
          ext_modules=[Extension('wWigIO',['external/KentLib/wWigIO/wWigIO.c'],extra_link_args=['-DMACHTYPE_x86_64','-lz','-lm','external/KentLib/lib/jkweb.a'],extra_compile_args='-w -shared -fPIC -p -Iexternal/KentLib/inc'.split(' ')),
                       Extension('wTwoBitIO',['external/KentLib/wTwoBitIO/wTwoBitIO.c'],extra_link_args=['-DMACHTYPE_x86_64','-lz','-lm','external/KentLib/lib/jkweb.a'],extra_compile_args='-w -shared -fPIC -p -Iexternal/KentLib/inc'.split(' '))],
          classifiers=['Environment :: Console',
                       'Development Status :: 3 - Alpha',
                       'Intended Audience :: Developers',
                       'License :: OSI Approved :: GNU General Public License (GPL)',
                       'License :: Free for non-commercial use',
                       'Operating System :: Unix',
                       'Programming Language :: Python :: 2.7',
                       'Topic :: Scientific/Engineering :: Bio-Informatics'],
          install_requires=install_requires)




#!/usr/bin/python
#Last-modified: 18 Mar 2014 03:56:03 PM

#         Module/Scripts Description
# 
# Copyright (c) 2008 Yunfei Wang <yfwang0405@gmail.com>
# 
# This code is free software; you can redistribute it and/or modify it
# under the terms of the BSD License (see the file COPYING included with
# the distribution).
# 
# @status:  experimental
# @version: 1.0.0
# @author:  Yunfei Wang
# @contact: yfwang0405@gmail.com

# ------------------------------------
# python modules
# ------------------------------------

from pylab import *
from wLib import IO

# ------------------------------------
# constants
# ------------------------------------

cdict  = {'red':  ((0.0, 0.0, 0.0),
                   (0.1, 0.0, 0.0),
                   (0.5, 1.0, 1.0),
                   (0.9, 1.0, 1.0),
                   (1.0, 0.8, 0.8)),

         'green': ((0.0, 0.0, 0.0),
                   (0.1, 0.0, 0.0),
                   (0.5, 1.0, 1.0),
                   (0.9, 0.0, 0.0),
                   (1.0, 0.0, 0.0)),

         'blue':  ((0.0, 0.8, 0.8),
                   (0.1, 1.0, 1.0),
                   (0.5, 1.0, 1.0),
                   (0.9, 0.0, 0.0),
                   (1.0, 0.0, 0.0))
        }

# ------------------------------------
# Misc functions
# ------------------------------------
def smooth(x,window_len=11,window='hanning'):
    '''Input is a matrix. Smooth the depth for each interval.'''
    nrow, ncol =x.shape
    if ncol < window_len:
        raise ValueError, "Input vector needs to be bigger than window size."
    if window_len<3:
        return x
    if not window in ['flat', 'hanning', 'hamming', 'bartlett', 'blackman']:
        raise ValueError, "Window is on of 'flat', 'hanning', 'hamming', 'bartlett', 'blackman'"
    #s=np.r_[2*x[0]-x[window_len-1::-1],x,2*x[-1]-x[-1:-window_len:-1]]
    if window == 'flat': #moving average
        w=mp.ones(window_len,'d')
    else:
        w=eval('np.'+window+'(window_len)')
    w/=w.sum()
    y = np.apply_along_axis( lambda row:np.convolve(w,row,mode='same'),1,x)
    return y

def standarize(data):
    # Data processing
    # remove outlier
    data = np.minimum(data, np.mean(data)+3*np.std(data)) # data > mean+3*std will be mean+3*std
    # smooth data
    data = smooth(data,11)
    # standarize
    data = (data-np.mean(data))/(np.max(data)-np.min(data))
    return data
    
def Plot(data, xticks=[], yticks=[], order=[], showColBar=False):
    nrow,ncol = data.shape
    # change the order of data
    if len(order) >0:
        data = data[order,]
    fig=figure(figsize=(2,8))
    axmatrix=fig.add_axes([0.18,0.1,0.65,0.8])
    plt.register_cmap(name='cust_cmap', data=cdict)
    im=axmatrix.matshow(data,aspect='auto',origin='lower',cmap=plt.get_cmap('Reds'))
    axmatrix.set_xticks([])
    axmatrix.set_yticks([])
    
    # Y axis
    aylabel=fig.add_axes([0.18,0.1,0.0,0.8])
    aylabel.set_yticks(yticks)
    aylabel.set_xticks([])
    
    # x axis
    axlabel=fig.add_axes([0.18,0.08,0.65,0.0])
    axlabel.set_xticks(xticks)
    axlabel.set_yticks([])
    
    #plot colorbar
    if showColBar:
        axcolor=fig.add_axes([0.86,0.1,0.03,0.8])
        colorbar(im,cax=axcolor)
    
    return fig

def ReadIntervals(bedfile):
    intervals =[]
    for tbed in IO.ColumnReader(sys.argv[1],ftype="bed"):
        intervals.append(tbed)
    return intervals
    
def ReadBigWig(intervals,wigfile):
    data = np.zeros((len(intervals),intervals[0].length()),dtype=float)
    for i,tbed in enumerate(intervals):
        data[i,:] += tbed.getWig(sys.argv[2],byPos=True)
    return data
    
# ------------------------------------
# Classes
# ------------------------------------

# ------------------------------------
# Main
# ------------------------------------

if __name__=="__main__":
    if len(sys.argv)==1:
        sys.exit("Example:"+sys.argv[0]+" *.bed *.bw *.pdf/png")
    # Read intervals
    intervals = ReadIntervals(sys.argv[1])
    data = ReadBigWig(intervals,sys.argv[2])
    
    # process data
    data = standarize(data)
    # plot data
    fig=Plot(data,[-2000,-1000,0],[])
    
    # Save figure
    fig.savefig(sys.argv[3],format=sys.argv[3].split(".")[1].upper())
    


